<?php

namespace App\Filament\Resources;

use App\Filament\Resources\AdditionalSettingResource\Pages;
use App\Filament\Resources\AdditionalSettingResource\RelationManagers;
use App\Models\AdditionalSetting;
use Filament\Forms;
use Filament\Forms\Form;
use Filament\Resources\Resource;
use Filament\Tables;
use Filament\Forms\Components\Tabs;
use Filament\Actions\Action;
use Filament\Forms\Components\FileUpload;
use Filament\Forms\Components\Radio;
use Filament\Forms\Components\Repeater;
use Filament\Forms\Components\Section;
use Filament\Forms\Components\Select;
use Filament\Forms\Components\Textarea;
use Filament\Forms\Components\TextInput;
use Filament\Forms\Concerns\InteractsWithForms;
use Filament\Forms\Contracts\HasForms;
use Illuminate\Database\Eloquent\Model;
use Filament\Notifications\Notification;
use Filament\Pages\Page;
use Filament\Support\Exceptions\Halt;
use Filament\Tables\Table;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\SoftDeletingScope;

class AdditionalSettingResource extends Resource
{
    protected static ?string $model = AdditionalSetting::class;

    protected static ?string $navigationIcon = 'heroicon-o-cog-6-tooth';
    protected static ?string $navigationGroup = 'Configurações';
    protected static ?string $navigationLabel = 'Configurações do site';
    protected static ?string $slug = 'configuracoes/gerais-do-site';
    protected static ?string $title = 'Configurações';
    protected static ?string $modelLabel = 'Configurações do site';
    public static function form(Form $form): Form
    {
        return $form->schema([
            Tabs::make('Informações')->tabs([
                Tabs\Tab::make('Cabeçalho')
                    ->schema([
                        FileUpload::make('header_logo')
                            ->label('Logo do cabeçalho')
                            ->image()
                            ->directory('settings/header'),
                    ]),
                Tabs\Tab::make('Rodapé')
                    ->schema([
                        FileUpload::make('footer_logo')
                            ->label('Logo do rodapé')
                            ->image()
                            ->directory('settings/footer'),
                        TextInput::make('copyright_text')
                            ->label('Copyright')
                            ->required(),
                    ]),
                Tabs\Tab::make('Redes sociais')
                    ->schema([
                        Repeater::make('social_networks')
                            ->label('Listagem das redes sociais')
                            ->schema([
                                Select::make('icon')
                                    ->label('Rede social')
                                    ->options([
                                        'fab fa-facebook-f' => 'Facebook',
                                        'fab fa-instagram' => 'Instagram',
                                        'fab fa-youtube' => 'Youtube',
                                        'fab fa-linkedin' => 'Linkedin',
                                    ])->required(),
                                TextInput::make('link')->url()->required(),
                            ])->addActionLabel('Adicionar rede social')->columns(2),
                    ]),
                    Tabs\Tab::make('Localização e contato')
                        ->schema([
                            Repeater::make('social_networks')
                                ->label('Listagem das redes sociais')
                                ->schema([
                                    Select::make('icon')
                                    ->label('Tipo')
                                    ->options([
                                        'fas fa-phone-alt' => 'Telefone',
                                        'fab fa-whatsapp' => 'Whatsapp',
                                        'far fa-envelope' => 'E-mail',
                                        'fas fa-map-marker-alt' => 'Endereço',
                                    ])->required(),
                                    TextInput::make('title')
                                        ->label('Título')
                                        ->string()
                                        ->required(),
                                    TextInput::make('link')
                                        ->label('Link')
                                        ->required(),
                                    Radio::make('target')
                                        ->label('Abrir link')
                                        ->inline()
                                        ->inlineLabel(false)
                                        ->default('_blank')
                                        ->options([
                                            '_blank' => 'Em nova aba',
                                            '_self' => 'Na aba atual',
                                        ]),
                            ])->addActionLabel('Adicionar contato')->columns(3),
                        ]),
                    Tabs\Tab::make('Informações adicionais')
                    ->schema([
                        FileUpload::make('placeholder_logo')
                                ->label('Imagem placeholder')
                                ->image()
                                ->directory('settings/placeholder'),
                        Repeater::make('social_networks')
                            ->label('Informações adicionais')
                            ->schema([
                                TextInput::make('copyright_text')
                                    ->label('Copyright')
                                    ->required()
                                    ->string(),
                                Textarea::make('footer_text')
                                    ->label('Texto do rodapé')
                                    ->string(),
                            ])
                            ->collapsible()
                            ->persistCollapsed(),
                            
                    ]),   
            ])->columnSpan('full'),
        
        ]);  
    }

    public static function getRelations(): array
    {
        return [
            //
        ];
    }

    public static function getPages(): array
    {
        return [
            'index' => Pages\EditAdditionalSetting::route('/'),
            
        ];
    }
     public static function canCreate(): bool
    {
        return false;
    }

    public static function canDelete(Model $record): bool
    {
        return false;
    }
    
}
