<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use RalphJSmit\Laravel\SEO\Support\HasSEO;
use RalphJSmit\Laravel\SEO\Support\SEOData;
use Spatie\Sluggable\HasSlug;
use Spatie\Sluggable\SlugOptions;

class Post extends Model
{
    use HasFactory;
    use HasSlug;
    use HasSEO;

    protected $fillable = [
        'title',
        'excerpt',
        'content',
        'featured_image',
        'status',
        'is_featured',
        'slug',
    ];

    protected $casts = [
        'status' => 'boolean',
        'is_featured' => 'boolean',
    ];

    public function getSlugOptions(): SlugOptions
    {
        return SlugOptions::create()
            ->generateSlugsFrom('title')
            ->saveSlugsTo('slug');
    }

    public function getRouteKeyName()
    {
        return 'slug';
    }

    public function categories()
    {
        return $this->belongsToMany(Category::class);
    }

    public function tags()
    {
        return $this->belongsToMany(Tag::class);
    }

    public function getPostThumbnailAttribute()
    {
        $thumbnail = asset('images/placeholder.jpg');
        if ($this->featured_image) {
            $thumbnail = asset("storage/{$this->featured_image}");
        }

        return $thumbnail;
    }

    public function getDynamicSEOData(): SEOData
    {
        return new SEOData(
            title: $this->seo->title ?? $this->title,
            description: $this->seo->description ?? $this->excerpt,
            image: asset("storage/{$this->featured_image}"),
        );
    }
}
